/****************************************************************************************** 
 *	Chili DirectX Framework Version 14.03.22											  *	
 *	Game.cpp																			  *
 *	Copyright 2014 PlanetChili.net <http://www.planetchili.net>							  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "Game.h"
#include "Mat3.h"
#include "TriangleStrip.h"
#include <array>
#include <random>


Game::Game( HWND hWnd,KeyboardServer& kServer,MouseServer& mServer )
	:
	gfx( hWnd ),
	kbd( kServer ),
	mouse( mServer ),
	port( gfx,{ 0,D3DGraphics::SCREENHEIGHT - 1,0,D3DGraphics::SCREENWIDTH - 1 } ),
	font(L"Arial",30.0f),
	tex(L"thumb_Bubble.png")
{
	Bubble::InitRNG();
	std::random_device rd;
	std::mt19937 rng( rd() );
	std::uniform_real_distribution<float> xDist( 50.0f,float( D3DGraphics::SCREENWIDTH - 51 ) );
	std::uniform_real_distribution<float> yDist( 50.0f,float( D3DGraphics::SCREENHEIGHT - 51 ) );
	const std::wstring kanji =
		L"邱Ƃł錴qFRg[bḧׂ̍ޗyѐH";

	for( const auto& c : kanji )
	{
		bubbles.emplace_back( std::wstring( 1,c ),Vec2{ xDist( rng ),yDist( rng ) },tex,font );
	}
}

Game::~Game()
{
}

void Game::Go()
{
#if NDEBUG
	const float dt = timer.GetTimeSec();
	timer.StartWatch();
#else
	const float dt = 1.0f / 60.0f;
#endif

	UpdateModel( dt );
	gfx.BeginFrame();
	ComposeFrame();
	gfx.EndFrame();
}

void Game::UpdateModel( float dt )
{
	for( Bubble& b : bubbles )
	{
		b.Update( dt );
	}
}

void Game::ComposeFrame()
{
	for( const Bubble& b : bubbles )
	{
		port.Draw( b.GetDrawable() );
	}
}