#pragma once
#include "Screen.h"
#include "Keyboard.h"
#include "GameScreen.h"
#include "Menu.h"

class OptionsScreen : public Screen
{
private:
	class OptionsMenu : public Menu
	{
	private:
		class ExitButton : public Button
		{
		public:
			ExitButton( StartMenu& m )
				:
				Button( m,L"Exit" ),
				m( m )
			{
			}
		private:
			virtual void OnPress() override
			{
				m.OnExit();
			}
		private:
			StartMenu& m;
		};
		class StartButton : public Button
		{
		public:
			StartButton( StartMenu& m )
				:
				Button( m,L"New Game" ),
				screen( m.parent )
			{}
		private:
			virtual void OnPress() override
			{
				screen.NewGame();
			}
		private:
			TitleScreen& screen;
		};
	public:
		StartMenu( TitleScreen& parent )
			:
			Menu( { 512,560 },400,15,40,{ 148,32,32,64 },
			L"Arial",BLACK,{ 128,128,255 },{ 52,52,72 },2 ),
			parent( parent )
		{
			AddItem( std::make_unique<StartButton>( *this ) );
			AddItem( std::make_unique<ExitButton>( *this ) );
			Finalize();
		}
	private:
		virtual void OnExit() override
		{
			parent.pMenu.release();
		}
	private:
		TitleScreen& parent;
	};
public:
	TitleScreen( D3DGraphics& gfx,KeyboardClient& kbd,ScreenContainer* ctr )
		:
		Screen( ctr ),
		timesFont( L"Times New Roman",100 ),
		arialFont( L"Arial",30 ),
		kbd( kbd ),
		gfx( gfx ),
		funk( D3DGraphics::SCREENWIDTH,D3DGraphics::SCREENHEIGHT )
	{
		Brush::SeedColor( rd() );
		brushes.reserve( nBrushes );
		for( int i = 0; i < nBrushes; i++ )
		{
			brushes.emplace_back( *this );
		}
	}
	virtual void Update( float dt ) override
	{
		Brush::MutateColor();
		for( Brush& b : brushes )
		{
			b.Step();
		}
	}
	virtual void Draw( D3DGraphics& gfx ) override
	{
		gfx.CopySurface( funk );
		gfx.DrawString( L"H.U.G.S.",{ 250.0f,300.0f },timesFont,BLACK );
		if( pMenu )
		{
			pMenu->Draw( gfx );
		}
		else
		{
			gfx.DrawString( L"PRESS ENTER",{ 380.0f,500.0f },arialFont,BLACK );
		}
	}
	virtual void HandleInput() override
	{
		const KeyEvent key = kbd.ReadKey();

		if( pMenu )
		{
			pMenu->HandleInput( key );
		}
		else if( key.GetCode() == VK_RETURN && key.IsPress() )
		{
			pMenu = std::make_unique<StartMenu>( *this );
		}
	}
private:
	void NewGame()
	{
		ChangeScreen( std::make_unique< CountScreen >(
			std::make_unique< GameScreen >( gfx,kbd,nullptr ),parent ) );
	}
private:
	D3DGraphics& gfx;
	KeyboardClient& kbd;
	Font timesFont;
	Font arialFont;
	Surface funk;
	std::random_device rd;
	std::vector< Brush > brushes;
	const int nBrushes = 50;
	std::unique_ptr<StartMenu> pMenu;
};